
/* eslint-disable no-use-before-define */

import { TypeComposer, graphql } from 'graphql-compose';
import NodeInterface from './nodeInterface';
import wrapMutationResolver from './wrapMutationResolver';
import { toGlobalId } from './globalId';
import { getNodeFieldConfig } from './nodeFieldConfig';

var GraphQLID = graphql.GraphQLID,
    GraphQLNonNull = graphql.GraphQLNonNull;

// all wrapped typeComposers with Relay, stored in this variable
// for futher type resolving via NodeInterface.resolveType method

export var typeMapForNode = {};
export var nodeFieldConfig = getNodeFieldConfig(typeMapForNode);

export function composeWithRelay(typeComposer) {
  if (!(typeComposer instanceof TypeComposer)) {
    throw new Error('You should provide TypeComposer instance to composeWithRelay method');
  }

  if (typeComposer.getTypeName() === 'Query' || typeComposer.getTypeName() === 'RootQuery') {
    typeComposer.setField('node', nodeFieldConfig);
    return typeComposer;
  }

  if (typeComposer.getTypeName() === 'Mutation' || typeComposer.getTypeName() === 'RootMutation') {
    // just skip
    return typeComposer;
  }

  if (!typeComposer.hasRecordIdFn()) {
    throw new Error('TypeComposer(' + typeComposer.getTypeName() + ') should have recordIdFn. ' + 'This function returns ID from provided object.');
  }

  var findById = typeComposer.getResolver('findById');
  if (!findById) {
    throw new Error('TypeComposer(' + typeComposer.getTypeName() + ') provided to composeWithRelay ' + 'should have findById resolver.');
  }
  typeMapForNode[typeComposer.getTypeName()] = {
    resolver: findById,
    tc: typeComposer
  };

  typeComposer.addFields({
    id: {
      type: new GraphQLNonNull(GraphQLID),
      description: 'The globally unique ID among all types',
      resolve: function resolve(source) {
        return toGlobalId(typeComposer.getTypeName(), typeComposer.getRecordId(source));
      }
    }
  });

  typeComposer.addInterface(NodeInterface);

  typeComposer.getResolvers().forEach(function (resolver, resolverName) {
    if (resolver.kind === 'mutation') {
      var wrappedResolver = wrapMutationResolver(resolver, {
        resolverName: resolverName,
        rootTypeName: typeComposer.getTypeName()
      });
      typeComposer.setResolver(resolverName, wrappedResolver);
    }
  });

  return typeComposer;
}