'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require('babel-runtime/core-js/object/define-property');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNodeFieldConfig = getNodeFieldConfig;

var _graphqlCompose = require('graphql-compose');

var _globalId = require('./globalId');

var _nodeInterface = require('./nodeInterface');

var _nodeInterface2 = _interopRequireDefault(_nodeInterface);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { (0, _defineProperty3.default)(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
/* eslint-disable no-param-reassign, import/prefer-default-export */


var GraphQLID = _graphqlCompose.graphql.GraphQLID,
    GraphQLNonNull = _graphqlCompose.graphql.GraphQLNonNull;

// this fieldConfig must be set to RootQuery.node field

function getNodeFieldConfig(typeMapForNode) {
  return {
    description: 'Fetches an object that has globally unique ID among all types',
    type: _nodeInterface2.default,
    args: {
      id: {
        type: new GraphQLNonNull(GraphQLID),
        description: 'The globally unique ID among all types'
      }
    },
    resolve: function resolve(source, args, context, info) {
      if (!args.id || !(typeof args.id === 'string')) {
        return null;
      }

      var _fromGlobalId = (0, _globalId.fromGlobalId)(args.id),
          type = _fromGlobalId.type,
          id = _fromGlobalId.id;

      if (!typeMapForNode[type]) {
        return null;
      }
      var findById = typeMapForNode[type].resolver;
      var tc = typeMapForNode[type].tc;

      if (findById && findById.resolve && tc) {
        var graphqlType = tc.getType();

        // set `returnType` to `info` for proper work of `getProjectionFromAST`
        // it will correctly add required fields for `relation` to `projection`
        var projection = void 0;
        if (info) {
          info.returnType = graphqlType;
          projection = (0, _graphqlCompose.getProjectionFromAST)(info);
        } else {
          projection = {};
        }

        // suppose that first argument is argument with id field
        var idArgName = (0, _keys2.default)(findById.args)[0];
        return findById.resolve({
          source: source,
          args: _defineProperty({}, idArgName, id), // eg. mongoose has _id fieldname, so should map
          context: context,
          info: info,
          projection: projection
        }).then(function (res) {
          if (!res) return res;
          res.__nodeType = graphqlType;
          return res;
        });
      }

      return null;
    }
  };
}